/*
 * @(#)WanSimulator.java	1.00 31st Mar 1999
 * 
 * Modification Log:
 */

import java.io.*;
import java.util.*;
import java.net.*;


/**
 * WanSimulator<br>
 * Do you need to test your software over a remote communication line where
 * response is slow? Typical VSAT connections have about 600ms delay between
 * transmissions. This may affect your applications performance if you access
 * the remote connection too frequently. Use this tool to simulate a VSAT
 * connection over your LAN. This is a proxy which puts a configurable delay
 * between every send and receive.<br>
 * It is very similar to my other utility - ProProxy. ProProxy can also be 
 * modified for this purpose and will probably perform better.
 * <br>
 * Usage:<br>
 * java WanSimulator <port> <proxy_port> [delay in ms]<br>
 * Delays are typically 600-1000 ms over a VSAT.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 14th Feb, 1999
 */

public class WanSimulator
{
	public static void main(String [] args)
	{
		int iListenPort, iProxyPort, iDelay=1000;

		if (args.length < 2)
		{
            System.out.println("Usage: java WanSimulator <port> <proxy_port> [delay in ms]\nDelays are typically 600-1000 ms over a vsat.");
			System.exit(0);
		}
	
		iListenPort = Integer.parseInt(args[0]);	
		iProxyPort = Integer.parseInt(args[1]);	
		if (args.length > 2)
		{
			iDelay = Integer.parseInt(args[2]);
		}

		System.out.println("ListenPort=" + iListenPort +
                                   " ProxyPort=" + iProxyPort);
		Proxy oneProxy = new Proxy(iListenPort, iProxyPort);
		System.out.println("Starting the Proxy...");
		oneProxy.setDelay(iDelay);
		oneProxy.createThread();
		System.out.println("Proxy started");
		while(oneProxy.bIsRunning)
		{
			try
			{
				Thread.sleep(5000);
			}
			catch(InterruptedException ie)
			{
				// ignore
			}
		}
		System.out.println("Proxy exited...");
		System.exit(0);
	}
}


class Proxy implements java.lang.Runnable
{
	int 					iListenPort, iProxyPort;
	public static Vector 	connStack = new Vector();
	Thread 					thisThread;
	int 					iDelay = 1000;

	public boolean	bIsRunning = true;

	public Proxy(int iListenPort, int iProxyPort)
	{
		this.iListenPort = iListenPort;
		this.iProxyPort = iProxyPort;
	}

	public void run()
	{
		ServerSocket serverSock=null;

		// listen for incoming connection. 
		// For each incoming connection start a new connection class
		try
		{
			System.out.println("Creating a server socket...");
			serverSock = new ServerSocket(iListenPort);
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}

		while(true)
		{
			try
			{
				System.out.println("Acepting on the server socket...");
				Socket conn = serverSock.accept();
				System.out.println("Got hooked...");
				ProxyConn proxyConn = new ProxyConn(conn, iProxyPort);
				System.out.println("New connection created");
				connStack.addElement(proxyConn);
				proxyConn.setDelay(iDelay);
				proxyConn.createThread();
				System.out.println("New connection started");
			}
			catch (IOException ioe)
			{
				ioe.printStackTrace();
				break;
			}
		}
		thisThread = null;
	}

	public void setDelay (int iDelay)
	{
		this.iDelay = iDelay;
	}
	
	public void createThread()
	{
		thisThread = new Thread(this);
		thisThread.start();
	}
}

class ProxyConn implements Runnable
{
	Socket 			sockIn, sockOut;
	InputStream 	inStreamClnt, inStreamSrvr;
	OutputStream 	outStreamClnt, outStreamSrvr;
	Thread 			thisThread;
	int 			iDelay = 1000;
	
	public ProxyConn(Socket sockIn, int iProxyPort)
	{
		this.sockIn = sockIn;
		try
		{
			System.out.println("Creating proxy connection...");
			sockOut = new Socket(InetAddress.getLocalHost(), iProxyPort);

			sockIn.setSoTimeout(100);
			sockOut.setSoTimeout(100);
			System.out.println("Getting all streams...");
			inStreamClnt = sockIn.getInputStream();
			inStreamSrvr = sockOut.getInputStream();
			outStreamClnt = sockIn.getOutputStream();
			outStreamSrvr = sockOut.getOutputStream();
		}
		catch(UnknownHostException uhe)
		{
			uhe.printStackTrace();
			// let it give NullPointerException
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			// let it give NullPointerException
		}
	}

	public void setDelay (int iDelay)
	{
		this.iDelay = iDelay;
	}

	public void run()
	{
		byte [] data = new byte[4096];
		int iNumBytesRead;
		boolean bContinue = true;

		while(bContinue)
		{
			try
			{
				Thread.sleep(iDelay);
			}
			catch (InterruptedException ie)
			{
				// ignore
			}

			System.out.println("Waiting for client data...");
			try
			{
				iNumBytesRead = inStreamClnt.read(data);

				if (iNumBytesRead < 0)
				{
					break;
				}
				else if(iNumBytesRead > 0)
				{
					outStreamSrvr.write(data, 0, iNumBytesRead);
				}
			}
			catch(InterruptedIOException ioe)
			{
				// ignore
			}
			catch (IOException ioe)
			{
				// break
				break;
			}


			System.out.println("Waiting for server data...");
			try
			{
				Thread.sleep(iDelay);
			}
			catch (InterruptedException ie)
			{
				// ignore
			}

			try
			{
				iNumBytesRead = inStreamSrvr.read(data);

				if (iNumBytesRead < 0)
				{
					break;
				}
				else if (iNumBytesRead > 0)
				{
					outStreamClnt.write(data, 0, iNumBytesRead);
				}
			}
			catch(InterruptedIOException ioe)
			{
				// ignore
			}
			catch (IOException ioe)
			{
				// break
				break;
			}
		}
	
		try
		{	
			System.out.println("Closing all streams...");
			sockIn.close();
			sockOut.close();
			inStreamClnt.close();
			inStreamSrvr.close();
			outStreamClnt.close();
			outStreamSrvr.close();
		}
		catch (IOException ioe)
		{
			// ignore
		}
		thisThread = null;
		System.out.println("Connection deregistered");
		Proxy.connStack.removeElement(this);
	}

	public void createThread()
	{
		thisThread = new Thread(this);
		thisThread.start();
	}
}

